﻿using System;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Shapes;
using Microsoft.Phone.Controls;
using Petzold.Phone.Silverlight;

namespace TouchInertiaDemo
{
    public partial class MainPage : PhoneApplicationPage
    {
        bool inertiaInProgress;
        bool abortInertia;
        Point absoluteEllipseLocation;

        public MainPage()
        {
            InitializeComponent();
            SupportedOrientations = SupportedPageOrientation.Portrait | SupportedPageOrientation.Landscape;
        }

        void OnPathManipulationStarted(object sender, ManipulationStartedEventArgs args)
        {
            if (inertiaInProgress)
                abortInertia = true;
        }

        void OnPathManipulationDelta(object sender, ManipulationDeltaEventArgs args)
        {
            SetVelocityTextBlock(args.Velocities.LinearVelocity);

            Path path = args.ManipulationContainer as Path;
            EllipseGeometry ellipseGeo = path.Data as EllipseGeometry;
            Point center = ellipseGeo.Center;
            Point translation = new Point();

            // Fix the stupid Orientation bug
            switch (this.Orientation)
            {
                case PageOrientation.PortraitUp:
                    translation = args.DeltaManipulation.Translation;
                    break;

                case PageOrientation.PortraitDown:
                    translation.X = -args.DeltaManipulation.Translation.X;
                    translation.Y = -args.DeltaManipulation.Translation.Y;
                    break;

                case PageOrientation.LandscapeLeft:
                    translation.X = args.DeltaManipulation.Translation.Y;
                    translation.Y = -args.DeltaManipulation.Translation.X;
                    break;

                case PageOrientation.LandscapeRight:
                    translation.X = -args.DeltaManipulation.Translation.Y;
                    translation.Y = args.DeltaManipulation.Translation.X;
                    break;
            }

            ellipseGeo.Center = new Point(center.X + translation.X, center.Y + translation.Y);
        }

        void OnPathManipulationCompleted(object sender, ManipulationCompletedEventArgs args)
        {
            SetVelocityTextBlock(args.FinalVelocities.LinearVelocity);

            // Save location of ellipse
            Path path = args.ManipulationContainer as Path;
            EllipseGeometry ellipseGeo = path.Data as EllipseGeometry;
            absoluteEllipseLocation = ellipseGeo.Center;

            // Create and initialize the InertiaManager
            InertiaManager inertiaManager = new InertiaManager(args);
            inertiaManager.ManipulationInertiaDelta += OnManipulationInertiaDelta;
            inertiaManager.ManipulationInertiaCompleted += OnManipulationInertiaCompleted;
            inertiaManager.TranslationBehavior.DesiredDeceleration = 100;

            // Begin inertia
            inertiaInProgress = true;
            abortInertia = false;
            inertiaManager.Start();
        }

        void OnManipulationInertiaDelta(object sender, ManipulationInertiaDeltaEventArgs args)
        {
            SetVelocityTextBlock(args.Velocities.LinearVelocity);

            if (abortInertia)
            {
                args.Complete();
                return;
            }

            Path path = args.ManipulationContainer as Path;
            EllipseGeometry ellipseGeo = path.Data as EllipseGeometry;

            absoluteEllipseLocation += args.DeltaManipulation.Translation;

            // Bounce it off the edges
            Size playground = new Size(ContentGrid.ActualWidth - 2 * ellipseGeo.RadiusX,
                                       ContentGrid.ActualHeight - 2 * ellipseGeo.RadiusY);

            double x = Math.Abs((absoluteEllipseLocation.X - ellipseGeo.RadiusX) % (2 * playground.Width));
            if (x > playground.Width)
                x = 2 * playground.Width - x;
            x += ellipseGeo.RadiusX;

            double y = Math.Abs((absoluteEllipseLocation.Y - ellipseGeo.RadiusY) % (2 * playground.Height));
            if (y > playground.Height)
                y = 2 * playground.Height - y;
            y += ellipseGeo.RadiusY;

            // Set the new ellipse center
            ellipseGeo.Center = new Point(x, y);
        }

        void OnManipulationInertiaCompleted(object sender, ManipulationInertiaCompletedEventArgs args)
        {
            SetVelocityTextBlock(args.FinalVelocities.LinearVelocity);
            inertiaInProgress = false;
        }

        void SetVelocityTextBlock(Vector velocity)
        {
            SetVelocityTextBlock((Point)velocity);
        }

        void SetVelocityTextBlock(Point velocity)
        {
            textBlockVelocity.Text = String.Format("Velocity = " + velocity);
        }
    }
}